@@ -17,10 +17,6 @@ BEGIN {
     @Test::More::EXPORT = grep { $_ ne 'explain' } @Test::More::EXPORT;
     Test::More->import;
 }
-use Test::Differences;
-use Test::Exception;
-use Test::Deep;
-use Test::Warn;
 
 use Test::Builder;
 my $OK_FUNC;
@@ -34,16 +30,40 @@ Test::Most - Most commonly needed test functions and features.
 
 =head1 VERSION
 
-Version 0.21
+Version 0.22
 
 =cut
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
+$VERSION = eval $VERSION;
 
 =head1 SYNOPSIS
 
-This module provides you with the most commonly used testing functions and
-gives you a bit more fine-grained control over your test suite.
+Instead of this:
+
+    use strict;
+    use warnings;
+    use Test::Exception 0.88;
+    use Test::Differences 0.500;
+    use Test::Deep 0.106;
+    use Test::Warn 0.11;
+    use Test::More tests => 42;
+
+You type this:
+
+    use Test::Most tests => 42;
+
+=head1 DESCRIPTION
+
+L<Test::Most> exists to reduce boilerplate and to make your testing life
+easier.  We provide "one stop shopping" for most commonly used testing
+modules.  In fact, we often require the latest versions so that you get bug
+fixes through L<Test::Most> and don't have to keep upgrading these modules
+separately.
+
+This module provides you with the most commonly used testing functions, along
+with automatically turning on strict and warning and gives you a bit more
+fine-grained control over your test suite.
 
     use Test::Most tests => 4, 'die';
 
@@ -55,6 +75,14 @@ gives you a bit more fine-grained control over your test suite.
 As you can see, the C<eq_or_diff> test will fail.  Because 'die' is in the
 import list, the test program will halt at that point.
 
+If you do not want strict and warnings enabled, you must explicitly disable
+them.  Thus, you must be explicit about what you want and no longer need to
+worry about accidentally forgetting them.
+
+    use Test::Most tests => 4;
+    no strict;
+    no warnings;
+
 =head1 EXPORT
 
 All functions from the following modules will automatically be exported into
@@ -62,15 +90,15 @@ your namespace:
 
 =over 4
 
-=item * C<Test::More>
+=item * L<Test::More>
 
-=item * C<Test::Exception>
+=item * L<Test::Exception>
 
-=item * C<Test::Differences>
+=item * L<Test::Differences>
 
-=item * C<Test::Deep> 
+=item * L<Test::Deep> 
 
-=item * C<Test::Warn>
+=item * L<Test::Warn>
 
 =back
 
@@ -130,10 +158,10 @@ of how C<Test::Builder> has been designed.
 
 =head2 C<explain>
 
-Similar to C<diag()>, but only outputs the message if C<$ENV{TEST_VERBOSE}> is
-set.  This is typically set by using the C<-v> switch with C<prove>.
+Similar to C<note()>, the output will only be seen by the user by
+using the C<-v> switch with C<prove> or reading the raw TAP.
 
-Unlike C<diag()>, any reference in the argument list is autumatically expanded
+Unlike C<note()>, any reference in the argument list is automatically expanded
 using C<Data::Dumper>.  Thus, instead of this:
 
  my $self = Some::Object->new($id);
@@ -157,7 +185,8 @@ C<$Indent>, C<Sortkeys> and C<Terse> all set to the value of C<1> (one).  This
 allows for a much cleaner diagnostic output and at the present time cannot be
 overridden.
 
-Requires C<Test::Harness> 3.07 or greater.
+Note that Test::More's C<explain> acts differently.  This C<explain>
+is equivalent to C<note explain> in Test::More.
 
 =head2 C<show>
 
@@ -189,8 +218,18 @@ only show the names for lexical variables.  Globals such as C<%ENV> or C<%@>
 are not accessed via PadWalker and thus cannot be shown.  It would be nice to
 find a workaround for this.
 
+=head2 C<always_explain> and C<always_show>
+
+These are identical to C<explain> and C<show>, but like L<Test::More>'s
+C<diag> function, these will always emit output, regardless of whether or not
+you're in verbose mode.
+
 =head2 C<all_done>
 
+B<DEPRECATED>.  Use the new C<done_testing()> (added in
+L<Test::More|Test::More> since 0.87_01).  Instead. We're leaving this in here
+for a long deprecation cycle.  After a while, we might even start warning.
+
 If the plan is specified as C<defer_plan>, you may call C<&all_done> at the
 end of the test with an optional test number.  This lets you set the plan
 without knowing the plan before you run the tests.
@@ -238,8 +277,37 @@ will restore the original behavior (not throwing an exception or bailing out).
 The C<die_on_fail> and C<bail_on_fail> functions will automatically set the
 desired behavior at runtime.
 
+=head2 Environment variables
+
+ DIE_ON_FAIL=1 prove t/
+ BAIL_ON_FAIL=1 prove t/
+
+If the C<DIE_ON_FAIL> or C<BAIL_ON_FAIL> environment variables are true, any
+tests which use C<Test::Most> will throw an exception or call BAIL_OUT on test
+failure.
+
+=head1 MISCELLANEOUS
+
+=head1 Excluding Test Modules
+
+Sometimes you want a exclude a particular test module.  For example,
+L<Test::Deep>, when used with L<Moose>, produces the following warning:
+
+    Prototype mismatch: sub main::blessed ($) vs none
+
+You can exclude this with by adding the module to the import list with a '-'
+symbol in front:
+
+    use Test::Most tests => 42, '-Test::Deep';
+
+See
+L<https://rt.cpan.org/Ticket/Display.html?id=54362&results=e73ff63c5bf9ba0f796efdba5773cf3f>
+for more information.
+
 =head2 Deferred plans
 
+B<DEPRECATED>.  Use C<done_testing()> from L<Test::More> instead.
+
  use Test::Most qw<defer_plan>;
  use My::Tests;
  my $test_count = My::Tests->run;
@@ -258,14 +326,22 @@ you I<ran> is the number of tests.  Until now, there was no way of asserting
 that the number of tests you I<expected> is the number of tests unless you do
 so before any tests have run.  This fixes that problem.
 
-=head2 Environment variables
+=head2 One-stop shopping
 
- DIE_ON_FAIL=1 prove t/
- BAIL_ON_FAIL=1 prove t/
+We generally require the latest stable versions of various test modules.  Why?
+Because they have bug fixes and new features.  You don't want to have to keep
+remembering them, so periodically we'll release new versions of L<Test::Most>
+just for bug 
 
-If the C<DIE_ON_FAIL> or C<BAIL_ON_FAIL> environment variables are true, any
-tests which use C<Test::Most> will throw an exception or call BAIL_OUT on test
-failure.
+=head2 C<use ok>
+
+We do not bundle L<Test::use::ok>, though it's been requested.  That's because
+C<use_ok> is broken, but L<Test::use::ok> is also subtly broken (and a touch
+harder to fix).  See L<http://use.perl.org/~Ovid/journal/39859> for more
+information.
+
+If you want to test if you can use a module, just use it.  If it fails, the
+test will still fail and that's the desired result.
 
 =head1 RATIONALE
 
@@ -274,23 +350,38 @@ hundreds of tests failing and whizzing by, you want the test suite to simply
 halt on the first failure.  This module gives you that control.
 
 As for the reasons for the four test modules chosen, I ran code over a local
-copy of the CPAN to find the most commonly used testing modules.  Here were
-the top ten (out of 287):
+copy of the CPAN to find the most commonly used testing modules.  Here's the
+top twenty as of January 2010 (the numbers are different because we're now
+counting distributions which use a given module rather than simply the number
+of times a module is used).
 
- Test::More              44461
- Test                     8937
- Test::Exception          1379
- Test::Simple              731
- Test::Base                316
- Test::Builder::Tester     193
- Test::NoWarnings          174
- Test::Differences         146
- Test::MockObject          139
- Test::Deep                127
+    1   Test::More                          14111
+    2   Test                                 1736
+    3   Test::Exception                       744
+    4   Test::Simple                          331
+    5   Test::Pod                             328
+    6   Test::Pod::Coverage                   274
+    7   Test::Perl::Critic                    248
+    8   Test::Base                            228
+    9   Test::NoWarnings                      155
+    10  Test::Distribution                    142
+    11  Test::Kwalitee                        138
+    12  Test::Deep                            128
+    13  Test::Warn                            127
+    14  Test::Differences                     102
+    15  Test::Spelling                        101
+    16  Test::MockObject                       87
+    17  Test::Builder::Tester                  84
+    18  Test::WWW::Mechanize::Catalyst         79
+    19  Test::UseAllModules                    63
+    20  Test::YAML::Meta                       61
 
-The four modules chosen seemed the best fit for what C<Test::Most> is trying
-to do.  As of 0.02, we've added L<Test::Warn> by request.  It's not in the top
-ten, but it's a great and useful module.
+L<Test::Most> is number 24 on that list, if you're curious.  See
+L<http://blogs.perl.org/users/ovid/2010/01/most-popular-testing-modules---january-2010.html>.
+
+The modules chosen seemed the best fit for what C<Test::Most> is trying to do.
+As of 0.02, we've added L<Test::Warn> by request.  It's not in the top ten, but
+it's a great and useful module.
 
 =cut
 
@@ -298,39 +389,32 @@ BEGIN {
     @ISA    = qw(Test::Builder::Module);
     @EXPORT = (
         @Test::More::EXPORT, 
-        @Test::Differences::EXPORT,
-        @Test::Exception::EXPORT,
-        @Test::Differences::EXPORT,
-        @Test::Deep::EXPORT,
-        @Test::Warn::EXPORT,
         qw<
             all_done
             bail_on_fail
             die_on_fail
             explain
+            always_explain
             last_test_failed
             restore_fail
             set_failure_handler
             show
+            always_show
         >
     );
-
-    if ( Test::Differences->VERSION <= 0.47 ) {
-
-        # XXX There's a bug in Test::Differences 0.47 which attempts to render
-        # an AoH in a cleaner 'table' format.
-        # http://rt.cpan.org/Public/Bug/Display.html?id=29732
-        no warnings 'redefine';
-        *Test::Differences::_isnt_HASH_of_scalars = sub {
-            return 1 if ref ne "HASH";
-            return scalar grep ref, values %$_;
-        };
-    }
 }
 
 sub import {
     my $bail_set = 0;
 
+    my %modules_to_load = map { $_ => 1 } qw/
+        Test::Differences
+        Test::Exception
+        Test::Deep
+        Test::Warn
+    /;
+    warnings->import;
+    strict->import;
     eval "use Data::Dumper::Names 0.03";
     $DATA_DUMPER_NAMES_INSTALLED = !$@;
 
@@ -349,33 +433,64 @@ sub import {
             last;
         }
     }
-    for my $i ( 0 .. $#_ ) {
+    my $i = 0;
+    while ($i < @_) {
         if ( !$bail_set and ( 'die' eq $_[$i] ) ) {
             splice @_, $i, 1;
             die_on_fail();
-            last;
+            $i = 0;
+            next;
         }
-    }
-    for my $i ( 0 .. $#_ ) {
-       if ( 'defer_plan' eq $_[$i] ) {
+        if ( $_[$i] =~ /^-(.*)/ ) {
+            my $module = $1;
+            splice @_, $i, 1;
+            unless (exists $modules_to_load{$module}) {
+                require Carp;
+                Carp::croak("Cannot remove non-existent Test::Module ($module)");
+            }
+            delete $modules_to_load{$module};
+            $i = 0;
+            next;
+        }
+        if ( 'defer_plan' eq $_[$i] ) {
             splice @_, $i, 1;
 
-           my $builder = Test::Builder->new;
-           $builder->{Have_Plan} = 1; # don't like setting this directly, but Test::Builder::has_plan doe
-           $builder->{TEST_MOST_deferred_plan} = 1;
-           $builder->{TEST_MOST_all_done} = 0;
-
-           last;
-       }
-   }
+            my $builder = Test::Builder->new;
+            $builder->{Have_Plan} = 1
+              ; # don't like setting this directly, but Test::Builder::has_plan doe
+            $builder->{TEST_MOST_deferred_plan} = 1;
+            $builder->{TEST_MOST_all_done}      = 0;
+            $i = 0;
+            next;
+        }
+        $i++;
+    }
+    foreach my $module (keys %modules_to_load) {
+        eval "use $module";
+        no strict 'refs';
+        push @EXPORT => @{"${module}::EXPORT"};
+        if ( my $error = $@) {
+            require Carp;
+            Carp::croak($error);
+        }
+    }
 
     # 'magic' goto to avoid updating the callstack
     goto &Test::Builder::Module::import;
 }
 
 sub explain {
-    return unless $ENV{TEST_VERBOSE};
-    Test::More::diag(
+    _explain(\&Test::More::note, @_);
+}
+
+sub always_explain {
+    _explain(\&Test::More::diag, @_);
+}
+
+sub _explain {
+    my $diag = shift;
+    no warnings 'once';
+    $diag->(
         map {
             ref $_
               ? do {
@@ -391,16 +506,24 @@ sub explain {
 }
 
 sub show {
-    return unless $ENV{TEST_VERBOSE};
+    _show(\&Test::More::note, @_);
+}
+
+sub always_show {
+    _show(\&Test::More::diag, @_);
+}
+
+sub _show {
     unless ( $DATA_DUMPER_NAMES_INSTALLED ) {
         warn "Data::Dumper::Names 0.03 not found.  Use explain() instead of show()";
-        goto &explain;
+        goto &_explain;
     }
+    my $diag = shift;
     no warnings 'once';
     local $Data::Dumper::Indent         = 1;
     local $Data::Dumper::Sortkeys       = 1;
-    local $Data::Dumper::Names::UpLevel = $Data::Dumper::Names::UpLevel + 1;
-    Test::More::diag(Data::Dumper::Names::Dumper(@_));
+    local $Data::Dumper::Names::UpLevel = $Data::Dumper::Names::UpLevel + 2;
+    $diag->(Data::Dumper::Names::Dumper(@_));
 }
 
 sub die_on_fail {
@@ -484,9 +607,11 @@ Curtis Poe, C<< <ovid at cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-test-extended at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-Most>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please report any bugs or feature requests to C<bug-test-extended at
+rt.cpan.org>, or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Test-Most>.  I will be
+notified, and then you'll automatically be notified of progress on your bug as
+I make changes.
 
 =head1 SUPPORT
 
